package com.qmth.themis.business.templete;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.qmth.themis.business.config.EnvConfig;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.entity.TBTaskHistory;
import com.qmth.themis.business.service.TBTaskHistoryService;
import com.qmth.themis.business.util.OssUtil;

import javax.swing.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Map;
import java.util.Objects;
import java.util.StringJoiner;

/**
 * @Description: 任务公用
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/19
 */
public class TaskImportCommon {
    private Integer mode = null;
    private Map<String, Object> map = null;
    private Long orgId = null;
    private Long createId = null;
    private Long examId = null;
    private TBTaskHistoryService tbTaskHistoryService;
    private String type;
    private String path;
    private Integer uploadType;
    private OssUtil ossUtil;
    private EnvConfig envConfig;
    private Map<String, Object> ossEnv;
    private TBTaskHistory tbTaskHistory;

    public TaskImportCommon(Map<String, Object> map) {
        this.map = map;
    }

    public void init() {
        Gson gson = new Gson();
        this.mode = Integer.parseInt(String.valueOf(this.map.get("mode")));
        this.orgId = Long.parseLong(String.valueOf(this.map.get("orgId")));
        this.createId = Long.parseLong(String.valueOf(this.map.get("createId")));
        this.examId = Long.parseLong(String.valueOf(this.map.get("examId")));
        JSONObject remarkJs = JSONObject.parseObject(String.valueOf(this.map.get("remark")));
        this.type = String.valueOf(remarkJs.get("type"));
        this.path = String.valueOf(remarkJs.get("path"));
        this.uploadType = Integer.parseInt(String.valueOf(remarkJs.get("uploadType")));
        this.ossUtil = SpringContextHolder.getBean(OssUtil.class);
        this.envConfig = SpringContextHolder.getBean(EnvConfig.class);
        this.ossEnv = this.envConfig.getOssEnv(this.uploadType);
        Map tbTaskHistoryMap = (Map) this.map.get("tbTaskHistory");
        tbTaskHistoryMap = SystemConstant.timeTransform(tbTaskHistoryMap);
        this.tbTaskHistory = gson.fromJson(gson.toJson(tbTaskHistoryMap), TBTaskHistory.class);
        this.tbTaskHistoryService = SpringContextHolder.getBean(TBTaskHistoryService.class);
    }

    /**
     * 获取上传的文件
     *
     * @return
     */
    public File getUploadFile() {
        StringJoiner localPath = new StringJoiner("").add(
                System.getProperty(SystemConstant.USER_DIR)).add(File.separator).add(path);
        File file = null;
        if (Objects.nonNull(this.type) && Objects.equals(this.type, SystemConstant.LOCAL)) {
            file = new File(localPath.toString());
        } else {
            try {
                file = this.ossUtil.ossDownload(this.ossEnv, localPath.toString(), localPath.toString());
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return file;
    }

    /**
     * 写入txt文件
     *
     * @param exception
     * @param txtStr
     * @throws IOException
     */
    public void writeImportResultTxt(boolean exception, String txtStr) throws IOException {
        FileWriter fileWriter = null;
        try {
            this.path = this.path.substring(0, this.path.lastIndexOf(File.separator) + 1);
            File file = new File(this.path + this.tbTaskHistory.getId() + ".txt");
            if (!file.exists()) {
                file.createNewFile();
            }
            fileWriter = new FileWriter(file);
            fileWriter.write(txtStr);
            JSONObject json = new JSONObject();
            json.put("path", file.getPath());
            json.put("type", this.type);
            if (exception) {
                this.tbTaskHistory.setErrorFilePath(json.toJSONString());
            } else {
                this.tbTaskHistory.setResultFilePath(json.toJSONString());
            }
            if (Objects.equals(this.type, SystemConstant.OSS)) {
                this.ossUtil.ossUpload(this.ossEnv, file.getPath(), txtStr);
                file.delete();
            }
            tbTaskHistoryService.updateById(this.tbTaskHistory);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(fileWriter)) {
                fileWriter.flush();
                fileWriter.close();
            }
        }
    }

    public Integer getMode() {
        return mode;
    }

    public Map<String, Object> getMap() {
        return map;
    }

    public Long getOrgId() {
        return orgId;
    }

    public Long getCreateId() {
        return createId;
    }

    public Long getExamId() {
        return examId;
    }

    public TBTaskHistoryService getTbTaskHistoryService() {
        return tbTaskHistoryService;
    }

    public String getType() {
        return type;
    }

    public String getPath() {
        return path;
    }

    public Integer getUploadType() {
        return uploadType;
    }

    public OssUtil getOssUtil() {
        return ossUtil;
    }

    public EnvConfig getEnvConfig() {
        return envConfig;
    }

    public Map<String, Object> getOssEnv() {
        return ossEnv;
    }

    public TBTaskHistory getTbTaskHistory() {
        return tbTaskHistory;
    }
}
